import torch 
import torch.nn as nn

from torch.distributions.normal import Normal 
from torch.distributions.multivariate_normal import MultivariateNormal
from torch.distributions.uniform import Uniform

class Gaussian_prior():
    
    def __init__(self, dim, device):
        
        self.dim = dim
        self.mean = nn.Parameter(torch.zeros(dim, device=device))
        self.cov = nn.Parameter(torch.eye(dim, device=device))
        self.device = device
        
    def get_params(self):
        return {self.mean, self.cov}
    
    def get_summary_statistics(self):
        return {'mean' : self.mean.data + torch.mm(self.cov, torch.zeros(self.dim, 1, device=self.device)).T, # mu + L * mu_x
                'cov' : torch.mm(self.cov, torch.mm(torch.eye(self.dim, device=self.device), self.cov.T))}    # L * sigma_x * L^T
        
    def sample(self, size):
        means = torch.zeros(size, self.dim, device=self.device)
        cov = torch.eye(self.dim, device=self.device)
        m = MultivariateNormal(means, cov)        
        return torch.bmm(m.sample().unsqueeze(1), torch.zeros(size, self.dim, self.dim, device=self.device) + self.cov).squeeze(1) + self.mean

    
class Uniform_prior():
    
    def __init__(self, dim, device):
        
        self.dim = dim
        self.lower_bound = nn.Parameter(torch.zeros(self.dim, device=device)) 
        self.higher_bound = nn.Parameter(torch.ones(self.dim, device=device)) 
        self.device = device
        
    def get_params(self):
        return {self.lower_bound, self.higher_bound}
    
    def get_summary_statistics(self):
        return {'lower_bound' : self.lower_bound.data,
                'higher_bound' : self.higher_bound.data}
        
    def sample(self, size):
        uniform = Uniform(torch.zeros(size, self.dim, device=self.device), torch.ones(size, self.dim, device=self.device))
        return uniform.sample() * (self.higher_bound - self.lower_bound) + self.lower_bound
