import matplotlib
import numpy
from matplotlib import pyplot as plt


def plot_empirical_prior_density(x, XLIM, YLIM, true_x=None, mle=None, bins=100, density=True, color_bar=True, simulator_name=None, do_plot=False, path_dir='figures/'):
    #assert x.shape[] == true_x.shape, "X and Y should have the same size"
    matplotlib.rcParams.update({'font.size': 14})
    
    for dim in range(int(numpy.ceil(x.shape[1]/2))):  
        
        dim1 = 2*dim if (2*dim+1)<x.shape[1] else 2*dim-1
        dim2 = 2*dim+1 if (2*dim+1)<x.shape[1] else 2*dim 

        plt.figure(figsize=(7, 7))
        plt.hist2d(x[:, dim1], x[:, dim2], bins=100, density=True, range=[XLIM, YLIM]);
        
        if color_bar:
            plt.colorbar()
        
        if true_x is not None:
            plt.scatter(true_x[dim1], true_x[dim2], c='red', marker='P', s=50, label='True parameters')
            
        if mle is not None:
            plt.scatter(mle[dim1], mle[dim2], c='w', marker='P', s=50, label='MLE')
        
        if (true_x is not None) or (mle is not None):
            plt.legend()
          
        plt.xlabel('x_' + str(dim1+1))
        plt.ylabel('x_' + str(dim2+1))
        plt.title('Learned Prior distribution $q_θ(x)$')
        
        if(simulator_name):
            plt.savefig(path_dir + simulator_name + '_prior_density_'+ str(dim1+1) + str(dim2+1) + '.png')
            
        if(do_plot):
            plt.show()
        else:
            plt.close()
            
            
def plot_empirical_posterior_density(x, y_likelihood, XLIM, YLIM, true_x=None, mle=None, bins=100, density=True, color_bar=True, simulator_name=None, do_plot=False, path_dir='figures/'):
    #assert x.shape[] == true_x.shape, "X and Y should have the same size"
    matplotlib.rcParams.update({'font.size': 14})
    
    for dim in range(int(numpy.ceil(x.shape[1]/2))):  
        
        dim1 = 2*dim if (2*dim+1)<x.shape[1] else 2*dim-1
        dim2 = 2*dim+1 if (2*dim+1)<x.shape[1] else 2*dim 

        plt.figure(figsize=(7, 7))
        plt.hist2d(x[:, dim1], x[:, dim2], bins=100, density=True, range=[XLIM, YLIM], weights=y_likelihood);
        
        if color_bar:
            plt.colorbar()
        
        if true_x is not None:
            plt.scatter(true_x[dim1], true_x[dim2], c='red', marker='P', s=50, label='True parameters')
            
        if mle is not None:
            plt.scatter(mle[dim1], mle[dim2], c='w', marker='P', s=50, label='MLE')
        
        if (true_x is not None) or (mle is not None):
            plt.legend()
          
        plt.xlabel('x_' + str(dim1+1))
        plt.ylabel('x_' + str(dim2+1))
        plt.title('Learned Posterior distribution $q_θ(x|y)$')
        
        if(simulator_name):
            plt.savefig(path_dir  + simulator_name + '_posterior_density_'+ str(dim1+1) + str(dim2+1) + '.png')
            
        if(do_plot):
            plt.show()
        else:
            plt.close()
            
            
def plot_data(x, true_x=None, mle=None, bins=100, simulator_name=None, do_plot=False, path_dir='figures/'):
    #assert x.shape[] == true_x.shape, "X and Y should have the same size"
    matplotlib.rcParams.update({'font.size': 14})
    
    for dim in range(int(numpy.ceil(x.shape[1]/2))):  
        
        dim1 = 2*dim if (2*dim+1)<x.shape[1] else 2*dim-1
        dim2 = 2*dim+1 if (2*dim+1)<x.shape[1] else 2*dim 

        plt.figure(figsize=(7, 7))
        plt.scatter(x[:, dim1], x[:, dim2], label='Learned distribution');
        
        if true_x is not None:
            plt.scatter(true_x[dim1], true_x[dim2], marker='P', s=50, label='True parameters')
            
        if mle is not None:
            plt.scatter(mle[dim1], mle[dim2], marker='P', s=50, label='MLE')
        
        if (true_x is not None) or (mle is not None):
            plt.legend()
          
        plt.xlabel('x_' + str(dim1+1))
        plt.ylabel('x_' + str(dim2+1))
        plt.title('Learned Prior distribution $q_θ(x)$')
        
        if(simulator_name):
            plt.savefig(path_dir + simulator_name + '_prior_'+ str(dim1+1) + str(dim2+1) + '.png')
    
        if(do_plot):
            plt.show()
        else:
            plt.close()
            
        
def regenerated_density_y(y, target=None, bins=100, simulator_name=None, do_plot=False, path_dir='figures/'):
    matplotlib.rcParams.update({'font.size': 14})
    
    for dim in range(int(numpy.ceil(y.shape[1]/2))):  
        
        dim1 = 2*dim if (2*dim+1)<y.shape[1] else 2*dim-1
        dim2 = 2*dim+1 if (2*dim+1)<y.shape[1] else 2*dim 

        plt.figure(figsize=(7, 7))
        plt.hist2d(y[:, dim1], y[:, dim2], bins=100, density=True);
        
        if target is not None:
            plt.scatter(target[dim1], target[dim2], c='red', marker='P', s=50, label='Target')
        
        if (target is not None):
            plt.legend()
          
        plt.xlabel('y_' + str(dim1+1))
        plt.ylabel('y_' + str(dim2+1))
        plt.title('y ~ p(y|x)')
        
        if(simulator_name):
            plt.savefig(path_dir + simulator_name + '_regenerated_y_density_'+ str(dim1+1) + str(dim2+1) + '.png')
    
        if(do_plot):
            plt.show()
        else:
            plt.close()
            
def regenerated_y(y, target=None, bins=100, simulator_name=None, do_plot=False, path_dir='figures/'):
    matplotlib.rcParams.update({'font.size': 14})
    
    for dim in range(int(numpy.ceil(y.shape[1]/2))):  
        
        dim1 = 2*dim if (2*dim+1)<y.shape[1] else 2*dim-1
        dim2 = 2*dim+1 if (2*dim+1)<y.shape[1] else 2*dim 

        plt.figure(figsize=(7, 7))
        plt.scatter(y[:, dim1], y[:, dim2], label='Regenerated y\'s');
        
        if target is not None:
            plt.scatter(target[dim1], target[dim2], marker='P', s=50, label='Target')
        
        if (target is not None):
            plt.legend()
          
        plt.xlabel('y_' + str(dim1+1))
        plt.ylabel('y_' + str(dim2+1))
        plt.title('y ~ p(y|x)')
        
        if(simulator_name):
            plt.savefig(path_dir + simulator_name + '_regenerated_y_'+ str(dim1+1) + str(dim2+1) + '.png')
    
        if(do_plot):
            plt.show()
        else:
            plt.close()