import numpy as np
import torch
from matplotlib import pyplot as plt
import matplotlib
from .utils import log_p_z
matplotlib.rcParams.update({'font.size': 14})

def plot_density(surrogate, target, prior_limits, plot_log_space=False, Nb_point_per_dimension = 100, cmap="summer", vmin=None, vmax=None, savepath=None):

    X_LIMS = prior_limits[0]
    Y_LIMS = prior_limits[1][::-1]

    x1 = np.linspace(*X_LIMS, Nb_point_per_dimension)
    x2 = np.linspace(*Y_LIMS, Nb_point_per_dimension)
    x1, x2 = np.meshgrid(x1, x2)
    x1 = x1.ravel()
    x2 = x2.ravel()

    x = np.c_[x1, x2]
    x = torch.FloatTensor(x)

    z_, log_jacobian = surrogate(torch.zeros(x.shape[0], target.shape[1]) + target, x)
    
    if plot_log_space:
        estimated_density = log_p_z(z_) + log_jacobian
    else:
        estimated_density = torch.exp(log_p_z(z_) + log_jacobian)
            
    estimated_density = estimated_density.data.numpy().reshape(Nb_point_per_dimension, Nb_point_per_dimension)

    fig = plt.figure(figsize=(7, 7))
    plt.imshow(estimated_density, extent=([X_LIMS[0], X_LIMS[1], Y_LIMS[1], Y_LIMS[0]]), cmap=cmap, vmin=vmin, vmax=vmax)
    if not plot_log_space:
        plt.title("$p(y_{obs}|x)$")
    else:
        plt.title("$\log p(y_{obs}|x)$")
    plt.ylabel('x_2')
    plt.xlabel('x_1')
    plt.colorbar()
    if savepath is not None:
        plt.savefig(savepath)
    plt.show()