import numpy as np
import torch
from torch.autograd import Variable
from matplotlib import pyplot as plt
import matplotlib
from .utils import log_p_z
matplotlib.rcParams.update({'font.size': 14})

def plot_gradient(surrogate, target, prior_limits, Nb_point_per_dimension = 50, out_of = 1, title=None, savepath=None):
    

    X_LIMS = prior_limits[0]
    Y_LIMS = prior_limits[1][::-1]
    Nb_point_per_dimension = 50

    x1 = np.linspace(*X_LIMS, Nb_point_per_dimension)
    x2 = np.linspace(*Y_LIMS, Nb_point_per_dimension)
    x1, x2 = np.meshgrid(x1, x2)
    x1 = x1.ravel()
    x2 = x2.ravel()

    x = np.c_[x1, x2]
    x = torch.FloatTensor(x)
    x = Variable(x, requires_grad=True)

    z_, log_jacobian = surrogate(torch.zeros(x.shape[0], target.shape[1]) + target, x)
    loss = (log_p_z(z_) + log_jacobian).mean()
    
    grad = torch.autograd.grad(loss, x)
    grad_x1 = grad[0][:, 0]
    grad_x2 = grad[0][:, 1]
    norm = torch.sqrt(grad_x1**2 + grad_x2**2)
    
    fig1, ax1 = plt.subplots()
    fig1.set_size_inches(18.5, 10.5)
    if title is not None:
        ax1.set_title(title)

    Q = ax1.quiver(x1[::out_of], x2[::out_of], grad_x1[::out_of].data.numpy()/norm[::out_of], grad_x2[::out_of].data.numpy()/norm[::out_of], norm[::out_of])

    '''plt.clim(0,150)'''
    fig1.colorbar(Q, extend='max')
    if savepath is not None:
        plt.savefig(savepath)
    plt.show()